### Sets

param price_levels integer >= 1;  # number of tiers

set TIERS = 1..price_levels;             
set ZONES;                               # Set of zones
set HOURS = 1..8760;                     # Set containing all hours of the year
set SUMMER = 3624..6551;                 # Hours corresponding to June 1 through September 30 in data
set PARTITION {TIERS} within HOURS;
set CUTS;                                # Defined to help satisfy capacity constraint 
set CUT_HOURS {CUTS} within SUMMER;

### Parameters

param cpp_days integer >= 0 default 0;   # Number of CPP days
param start integer >= 0 default 7;      # Hour in which higher TOU price starts
param end integer >= 0 default 22;       # Hour in which higher TOU price ends

param load_hist{HOURS,ZONES} >= 0;       # Historical load

param price_clearing_hist{HOURS};        # Historical clearing price for energy
param holiday_weekend{HOURS};            # Indicator that an hour falls on a holiday or weekend day
param cpp_rank{HOURS};                   # Pre-defined ordering of critical peak periods
param time_of_day{HOURS};                # Hour of day

param capacity_cost >= 0 default 120;                    # $/MW-day
param capacity_zone_hist{ZONES} >= 0;                    # MW, attributable to zone
param capacity_rto_baseline >= 0;                        # MW, including reserve
param capacity_reserve_factor >= 1 default 1.121934619;  # Default reserve factor calculated from historical data

param demand_elas <= 0 default -0.05;                    # Demand elasticity
param distribution_cost >= 0 default 15;                 # $/MWh
param supply_cost_intercept default 0;                   # $/MWh
param supply_scaling_factor default 18129;               # Scaling factor from supply model           
param supply_elas default 2.7424;                        # Default supply elasticity 

param variable_rate{ZONES} binary default 0;             # 1 if zone is included in variable rate, 0 otherwise

### Calculated parameters

param capacity_zone_var = sum{j in ZONES} variable_rate[j]*capacity_zone_hist[j]; # MW, attributable to variable_rate zones

param load_var{i in HOURS} = sum{j in ZONES} variable_rate[j]*load_hist[i,j]  ; # historical load for variable_rate zones

param load_fixed{i in HOURS} = sum{j in ZONES} (1 - variable_rate[j])*load_hist[i,j]; # historical load for excluded zones

param price_baseline = (sum{i in HOURS} (load_var[i]*price_clearing_hist[i])) 
                   / (sum{i in HOURS} load_var[i])
				   +
				   365 * capacity_cost * capacity_zone_var
				   / sum{i in HOURS} load_var[i]
				   + distribution_cost;
				   
param demand_coef{i in HOURS} = load_var[i] / (price_baseline^demand_elas);

param supply_coef{i in HOURS} = (price_clearing_hist[i] - supply_cost_intercept)
                              /((load_var[i] + load_fixed[i])/supply_scaling_factor)^supply_elas;

### Decision variables

var Price{TIERS} >= 0;
var Price_Hourly{HOURS} >= 0;
var Cost_Hourly{HOURS};
var Load_Hourly{HOURS};
var Consumer_Surplus;
var Retailer_Surplus;

var Z; # 5CP load

### Objective function

maximize Global_Surplus:
    Consumer_Surplus + Retailer_Surplus;
	
### Constraints
			    
subject to Tier{i in TIERS, j in PARTITION[i]}:
   Price_Hourly[j] = Price[i];
				
subject to Demand{i in HOURS}:
   Load_Hourly[i] = demand_coef[i]*(Price_Hourly[i]+distribution_cost)^demand_elas;

subject to Cost{i in HOURS}:
   Cost_Hourly[i] = supply_cost_intercept + supply_coef[i]*((load_fixed[i] + Load_Hourly[i])/supply_scaling_factor)^supply_elas;
   
subject to Benefit_Calc:
   Consumer_Surplus = sum{i in HOURS} 
				 ( 
					demand_coef[i] * 
					 ( price_baseline^(demand_elas+1) 
					   - (Price_Hourly[i]+distribution_cost)^(demand_elas+1)
					 ) / (demand_elas + 1)
				 );
   
subject to Profit_Calc:
   Retailer_Surplus = sum{i in HOURS}
				 (
				   (Price_Hourly[i] - Cost_Hourly[i]) * Load_Hourly[i]
				 )	   
				 - 365 * capacity_cost * capacity_zone_var
				 - 365 * capacity_cost * (Z * capacity_reserve_factor - capacity_rto_baseline);
      
subject to Peak{i in CUTS}:
   sum{j in CUT_HOURS[i]} (Load_Hourly[j] + load_fixed[j]) <= 5 * Z;
   
subject to Profitability:
   Retailer_Surplus <= 0;
   
subject to Clearing1{i in HOURS}:
   Price_Hourly[i] <= Cost_Hourly[i];

subject to Clearing2{i in HOURS}:
   Price_Hourly[i] >= Cost_Hourly[i];